#include "service.h"

// service tenants constructor with arguments
ServiceTenants::ServiceTenants(RepositoryAbstract& repository, Valid valid)
: repository(repository)
, valid(valid)
{
}

// service destructor
ServiceTenants::~ServiceTenants()
{
	for (const auto& undo : undoStack)
		delete undo;
}

// adds tenant
void ServiceTenants::add(int number, int surface,
std::string type, std::string name)
{
	Tenant t(number, surface, type, name);
	valid.validate(t);
	repository.add(t);
	UndoAction* undo = new UndoAdd(repository, t);
	undoStack.push_back(undo);
}

// removes tenant using id
void ServiceTenants::remove(int number)
{
	UndoAction* undo = new UndoRemove(repository, repository.search(number));
	undoStack.push_back(undo);
	repository.remove(number);
}	

// modifies tenant
void ServiceTenants::modify(int number, int surface,
std::string type, std::string name)
{
	UndoAction* undo = new UndoModify(repository, repository.search(number));
	undoStack.push_back(undo);
	repository.remove(number);
	Tenant t(number, surface, type, name);
	valid.validate(t);
	repository.add(t);
}

// searches tenant using id
const Tenant& ServiceTenants::search(int number)
{
	return repository.search(number);
}

// returns all tenants as std::vector
std::vector<Tenant> ServiceTenants::getTenants()
{
	return repository.getTenants();
}

// filter type for tenants
std::vector<Tenant> ServiceTenants::filterType(const std::string& type)
{
	std::vector<Tenant> tenants, tenant = repository.getTenants();
	std::copy_if(tenant.begin(), tenant.end(), std::back_inserter(tenants),
		[type](const Tenant& tenant)
		{
			return tenant.getType() == type;
		}
	);
	return tenants;
}

// filter surface for tenants
std::vector<Tenant> ServiceTenants::filterSurface(const int& surface)
{
	
	std::vector<Tenant> tenants, tenant = repository.getTenants();
	std::copy_if(tenant.begin(), tenant.end(), std::back_inserter(tenants),
		[surface](const Tenant& tenant)
		{
			return tenant.getSurface() == surface;
		}
	);
	return tenants;
}

// sort name for tenants
std::vector<Tenant> ServiceTenants::sortName()
{
	std::vector<Tenant> tenants = repository.getTenants();
	std::sort(tenants.begin(), tenants.end(),
		[](const Tenant& first, const Tenant& second)
		{
			return first.getName() < second.getName();
		}
	);
	return tenants;
}

// sort surface for tenants
std::vector<Tenant> ServiceTenants::sortSurface()
{
	std::vector<Tenant> tenants = repository.getTenants();
	std::sort(tenants.begin(), tenants.end(),
		[](const Tenant& first, const Tenant& second)
		{
			return first.getSurface() < second.getSurface();
		}
	);
	return tenants;
}

// sort type and surface for tenants
std::vector<Tenant> ServiceTenants::sortTypeSurface()
{
	std::vector<Tenant> tenants = repository.getTenants();
	std::sort(tenants.begin(), tenants.end(),
		[](const Tenant& first, const Tenant& second)
		{
			if (first.getType() == second.getType())
				return first.getSurface() < second.getSurface();
			return first.getType() < second.getType();
		}
	);
	return tenants;
}

// add number to cart
void ServiceTenants::addCart(const unsigned int& i)
{
	cart.add(repository.search(i));
}

// clear cart
void ServiceTenants::clearCart()
{
	cart.clear();
}

// get cart
const std::vector<Tenant>& ServiceTenants::getCart() const
{
	return cart.get();
}

// random cart
void ServiceTenants::randomCart(const unsigned int& size)
{
	cart.random(size, repository.getTenants());
}

// return map report
std::map<int, int> ServiceTenants::mapReport()
{
	std::map<int, int> report;
	std::vector<Tenant> tenants = repository.getTenants();
	for (const auto& tenant : tenants)
		if (report.find(tenant.getSurface()) != report.end())
			++report[tenant.getSurface()];
		else
			report.insert({tenant.getSurface(), 1});
	return report;
}

// undo
void ServiceTenants::undo()
{
	if (undoStack.empty())
		return;
	
	UndoAction* undo = undoStack.back();
	undoStack.pop_back();
	undo->undo();
	delete undo;
}

// export html
void ServiceTenants::exportHTML(const std::string& name)
{
	eHTML(name, cart.get());
}

// export csv
void ServiceTenants::exportCSV(const std::string& name)
{
	eCSV(name, cart.get());
}